/*
###################################
#     2003 Ivan 'Rig' Tsiroulev	  #
#		-=FRT=-					  #
###################################
*/

#include <windows.h>

typedef struct 
	{
	BYTE direction;
	int  keyLen;
	char keyMaterial[68];
	DWORD keySig;
	int	  numRounds;
	DWORD key32[8];
	DWORD sboxKeys[4];
	DWORD subKeys[40];
	} keyInstance;
typedef struct 
	{
	BYTE  mode;
	BYTE  IV[16];
	DWORD cipherSig;
	DWORD iv32[4];
	} cipherInstance;

#define	p8(N)	P8x8[P_##N]
#define	RS_rem(x)		\
	{ BYTE  b  = (BYTE) (x >> 24);											 \
	  DWORD g2 = ((b << 1) ^ ((b & 0x80) ? 0x14D : 0 )) & 0xFF;		 \
	  DWORD g3 = ((b >> 1) & 0x7F) ^ ((b & 1) ? 0x14D >> 1 : 0 ) ^ g2 ; \
	  x = (x << 8) ^ (g3 << 24) ^ (g2 << 16) ^ (g3 << 8) ^ b;				 \
	}
#define	LFSR1(x) (((x)>>1)^(((x)&0x01)?0x169/2 : 0))
#define	LFSR2(x) (((x)>>2)^(((x)&0x02)?0x169/2:0)^(((x) & 0x01)?0x169/4:0))
#define	Mx_1(x) ((DWORD)  (x))
#define	Mx_X(x) ((DWORD) ((x)^LFSR2(x)))
#define	Mx_Y(x) ((DWORD) ((x)^LFSR1(x)^LFSR2(x)))
#define	M00		Mul_1
#define	M01		Mul_Y
#define	M02		Mul_X
#define	M03		Mul_X
#define	M10		Mul_X
#define	M11		Mul_Y
#define	M12		Mul_Y
#define	M13		Mul_1
#define	M20		Mul_Y
#define	M21		Mul_X
#define	M22		Mul_1
#define	M23		Mul_Y
#define	M30		Mul_Y
#define	M31		Mul_1
#define	M32		Mul_Y
#define	M33		Mul_X
#define	Mul_1	Mx_1
#define	Mul_X	Mx_X
#define	Mul_Y	Mx_Y
#define	P_00	1
#define	P_01	0
#define	P_02	0
#define	P_03	(P_01^1)
#define	P_04	1
#define	P_10	0
#define	P_11	0
#define	P_12	1
#define	P_13	(P_11^1)
#define	P_14	0
#define	P_20	1
#define	P_21	1
#define	P_22	0
#define	P_23	(P_21^1)
#define	P_24	0
#define	P_30	0
#define	P_31	1
#define	P_32	1
#define	P_33	(P_31^1)
#define	P_34	1
#define	ROL(x,n) (((x) << ((n) & 0x1F)) | ((x) >> (32-((n) & 0x1F))))
#define	ROR(x,n) (((x) >> ((n) & 0x1F)) | ((x) << (32-((n) & 0x1F))))
#define		Bswap(x)			(x)
#define	_b(x,N)	(((BYTE *)&x)[((N) & 3)^0])
#define CRYPT_GAMEKEY_LENGTH    6
#define CRYPT_GAMEKEY_COUNT     25
#define CRYPT_GAMESEED_LENGTH   8
#define CRYPT_GAMESEED_COUNT    25
#define CRYPT_GAMETABLE_START   1
#define CRYPT_GAMETABLE_STEP    3
#define CRYPT_GAMETABLE_MODULO  11
#define CRYPT_GAMETABLE_TRIGGER 21036

class CCryptLogin
{
public:
	CCryptLogin() {	};
	virtual ~CCryptLogin() { };

	void  Decrypt(BYTE* pOutput, const BYTE* pInput, int iLen);
    void  init(BYTE * pseed, DWORD k1, DWORD k2);

protected:
	DWORD m_MasterHi;
    DWORD m_MasterLo;
    
    DWORD m_CryptMaskHi;
    DWORD m_CryptMaskLo;
};

class GameCrypt
{
public:
    GameCrypt() {};
    virtual ~GameCrypt() {};
	virtual void decrypt(BYTE * in, BYTE * out, int len) = 0;
    virtual void encrypt(BYTE * in, BYTE * out, int len) = 0;
    virtual void init(BYTE IP[4]=NULL) = 0;
};

class OldGameCrypt : public GameCrypt
{
private:
    static bool m_tables_ready;

    BYTE m_seed[CRYPT_GAMESEED_LENGTH];
    int m_table_index;
    int m_block_pos;
    int m_stream_pos;

    void init_tables();
    void raw_encrypt(UINT* values, int table);

public:
    OldGameCrypt();
    virtual ~OldGameCrypt();
    virtual void encrypt(BYTE * in, BYTE * out, int len);
    virtual void decrypt(BYTE * in, BYTE * out, int len);
    virtual void init(BYTE IP[4]=NULL);
};

class NewGameCrypt : public GameCrypt
{
private:
    unsigned long	m_IP;
    int				m_pos;
    keyInstance		ki;
    cipherInstance	ci;
    BYTE			m_subData3[256];
    DWORD			dwIndex;
	int				tabEnable;
	BYTE			tabUsed[256];
	int				numRounds[4];

public:
    NewGameCrypt();
    ~NewGameCrypt();
    virtual void encrypt(BYTE * in, BYTE * out, int len);
	virtual void decrypt(BYTE * in, BYTE * out, int len) { encrypt(in, out, len);};
    virtual void init(BYTE IP[4]);

protected:
	void makeKey(keyInstance *key, BYTE direction, int keyLen, char *keyMaterial);
	void cipherInit(cipherInstance *cipher, BYTE mode, char *IV);
	void blockEncrypt(cipherInstance *cipher, keyInstance *key, BYTE *input,	int inputLen, BYTE *outBuffer);
	void reKey(keyInstance *key);
	DWORD RS_MDS_Encode(DWORD k0,DWORD k1);
	DWORD f32(DWORD x, DWORD *k32,int keyLen);
	static BYTE P8x8[2][256];
};

class BothGameCrypt : public GameCrypt
{
private:
	OldGameCrypt* m_pOldGameCrypt;
	NewGameCrypt* m_pNewGameCrypt;

public:
    BothGameCrypt() 
	{
		m_pOldGameCrypt = new OldGameCrypt();
		m_pNewGameCrypt = new NewGameCrypt();
	};
    virtual ~BothGameCrypt()
	{
		delete m_pOldGameCrypt;
		delete m_pNewGameCrypt;
	};
    virtual void encrypt(BYTE * in, BYTE * out, int len)
	{
		m_pOldGameCrypt->encrypt(in, out, len);
		m_pNewGameCrypt->encrypt(out, out, len);
	};
    virtual void decrypt(BYTE * in, BYTE * out, int len)
	{
		m_pNewGameCrypt->decrypt(in, out, len);
		m_pOldGameCrypt->decrypt(out, out, len);

	};
    virtual void init(BYTE IP[4])
	{
		m_pOldGameCrypt->init();
		m_pNewGameCrypt->init(IP);
	};
};